import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author Suat
 * 
 *         This class is responsible for transforming the given proof's file in
 *         tex codes using the standards of the package boxproof.sty
 * 
 * 
 * */
public class LinearView {

	/**
	 * @param equality
	 *            is the whole line and it is splitted than the equality by the
	 *            given line
	 * @return returns the given equality in tex code using the package
	 *         boxproof.sty
	 */
	public String approx(String equality) {
		String approxString = null;
		Pattern approx = Pattern.compile(";.+;");

		Matcher matchApprox = approx.matcher(equality);
		while (matchApprox.find())
			approxString = "{"
					+ matchApprox.group(0).replaceAll("=", " \\\\approx ")
							.replaceAll(";", "") + "}";
		return approxString;

	}

	/**
	 * @param in
	 *            is the list which contains the the given proof file.
	 * @return returns the list which contains the given proof file's tex
	 *         codes.here it is transformed line by line
	 */
	public ArrayList<String> pline(ArrayList<String> in) {
		ArrayList<String> list = new ArrayList<String>();
		for (String s : in) {

			String rule = rulesParts(s)[1];
			int line = Integer.parseInt(rulesParts(s)[0]);

			if (rule.matches("prem"))
				list.add("\\pline[" + line + "]" + approx(s)
						+ "{\\text{prem}}\n");
			else if (s.contains("cong")) {
				String congPref = "";

				for (int i = 0; i < s.split(";")[2].split("cong")[1].split(",").length; i++)
					if (i == s.split(";")[2].split("cong")[1].split(",").length - 1) {
						// replaceAll("\\r", "")=split("$")[0];//last prefnumber
						// has Carriage return
						String end = s.split(";")[2].split("cong")[1]
								.split(",")[i].replaceAll("\\r", "");

						congPref += "\\pref{" + end + "}";

					} else
						congPref += "\\pref{"
								+ s.split(";")[2].split("cong")[1].split(",")[i]
								+ "}, ";
				list.add("\\pline[" + line + "]" + approx(s)
						+ "{\\text{{cong} " + congPref + "}}\n");

			} else if (rule.matches("trans\\d+,\\d+")) {
				int l = Integer.parseInt(rulesParts(s)[3]);
				int r = Integer.parseInt(rulesParts(s)[4]);

				list.add("\\pline[" + line + "]" + approx(s)
						+ "{\\text{{trans} \\pref{" + l + "}, " + "\\pref{" + r
						+ "}}}\n");
			} else if (rule.matches("sym\\d+")) {
				int ref = Integer.parseInt(rulesParts(s)[2]);

				list.add("\\pline[" + line + "]" + approx(s) + "{\\text{"
						+ "sym" + " " + "\\pref{" + ref + "}}}\n");

			} else if (rule.matches("app\\d+")) {
				int ref = Integer.parseInt(rulesParts(s)[2]);

				list.add("\\pline[" + line + "]" + approx(s) + "{\\text{"
						+ "app" + " " + "\\pref{" + ref + "}}}\n");
			} else if (rule.matches("ref"))
				list.add("\\pline[" + line + "]" + approx(s)
						+ "{\\text{ ref}}\n");

		}
		return list;

	}

	/**
	 * @param in
	 *            is the given proof file.The given proof file transformed in
	 *            tex code via calling the method pline. And the added empty
	 *            line ignored.it started for reading at the line 1 not at the
	 *            line 0
	 * @return returns
	 * 
	 *         returns the whole transformed file for the linear-view
	 */
	public ArrayList<String> linearview(ArrayList<String> in) {
		ArrayList<String> list = new ArrayList<String>();
		ArrayList<String> temp = new ArrayList<String>();

		startProofFile(list);
		startProofCode(list);

		for (int i = 1; i < in.size(); i++)
			temp.add(in.get(i));
		for (String string : pline(temp))
			list.add(string);

		endProofCode(list);
		endProofFile(list);

		return list;
	}

	/**
	 * @param string
	 *            is the line which is in the given proof file.
	 * @return
	 * 
	 *         returns an array of a String.the array contains as follows:
	 *         rulesInParts[0]= is the number/index of every line
	 *         rulesInParts[1]= is the name of the rule..........
	 *         rulesInParts[2]= is the reference-number/argument of the rules
	 *         symetrity,application and congruence with a argument
	 *         rulesInParts[3]= the first argument of trans.........
	 *         rulesInParts[4]= the second argument of trans ..........
	 *         rulesInParts[5]= is same of rulesInParts[1] ............
	 *         rulesInParts[6]= the first argument of cong with two arguments
	 *         rulesInParts[7]= the secong argument of cong with tow arguments
	 * 
	 */
	public String[] rulesParts(String string) {
		String[] rulesInParts = new String[8];
		// 0=linenum, 1=rule, 2=proofrefnum of all rules except trans, 3=first
		// proofrefnum of trans, 4=secondproofrefnum of trans
		// find line number 6,7 cong
		Pattern UnaryFirstNum = Pattern.compile("\\d+;");
		Matcher UnaryNum = UnaryFirstNum.matcher(string);
		while (UnaryNum.find())
			rulesInParts[0] = UnaryNum.group(0).replaceAll(";", "");

		Pattern pattern = Pattern
				.compile("cong\\d+,\\d+|sym\\d+|cong\\d+|prem|app\\d+|app|ref|trans\\d+,\\d+");
		Matcher matcher = pattern.matcher(string);
		while (matcher.find())
			// if(matcher.group().equals("prem"))
			// plineRef[1]= matcher.group(0).replaceAll("prem", "app");
			// else
			// rulesInParts[1] = matcher.group(0).replaceAll("\\d+|,", "");
			rulesInParts[1] = matcher.group(0);
		// find prooflinerefnumbers except trans
		Pattern UnarySecondNum = Pattern
				.compile("cong\\d+|cong\\d+|app\\d+|sym\\d+");
		Matcher UnarySecNum = UnarySecondNum.matcher(string);
		while (UnarySecNum.find())
			rulesInParts[2] = UnarySecNum.group(0).replaceAll("sym|cong|app",
					"");

		// find prooflinerefnumbers for trans
		Pattern trans = Pattern.compile("trans\\d+,\\d+");
		Matcher transpart = trans.matcher(string);
		while (transpart.find()) {

			// 11.03 added \\d?
			// plineTrans[0] = UnaryNum.group(0).replaceAll("\\d\\d?,\\d\\d?",
			// "");
			rulesInParts[3] = transpart.group(0).replaceAll("trans|,\\d+", "");// firstnumber
			// of
			// trans
			rulesInParts[4] = transpart.group(0).replaceAll("trans\\d+,", "");// secondnumber
			// of
			// trans

		}
		// dublicate of rulesInParts[1], hence useles
		Pattern pattern2 = Pattern
				.compile("cong\\d+|cong\\d|sym\\d+|prem|app\\d+|app|trans\\d+,\\d+");
		Matcher matcher5 = pattern2.matcher(string);
		while (matcher5.find())
			// if(matcher.group().equals("prem"))
			// plineRef[1]= matcher.group(0).replaceAll("prem", "app");
			// else
			// rulesInParts[5] = matcher2.group(0).replaceAll("\\d+|,", "");
			rulesInParts[5] = matcher5.group(0);

		// find prooflinerefnumbers for cong
		Pattern cong = Pattern.compile("cong\\d+,\\d+");
		Matcher congpart = cong.matcher(string);
		while (congpart.find()) {

			// 11.03 added \\d?
			// plineTrans[0] = UnaryNum.group(0).replaceAll("\\d\\d?,\\d\\d?",
			// "");
			rulesInParts[6] = congpart.group(0).replaceAll("cong|,\\d+", "");// firstnumber
			// of
			// cong
			rulesInParts[7] = congpart.group(0).replaceAll("cong\\d+,", "");// secondnumber
			// of
			// cong

		}

		return rulesInParts;

	}

	/**
	 * @param list
	 *            is the list which contains the whole tex codes. And to this
	 *            list is added the tex codes for starting the transformed tex
	 *            file.
	 */
	public void startProofFile(ArrayList<String> list) {
		list.add("\\documentclass{article}\n\\usepackage{boxproof}\n\\usepackage{amsmath}\n\\begin{document}\n");

	}

	/**
	 * @param list
	 *            is the list which contains the whole tex codes. and to this
	 *            list is added the tex codes which we need for starting the
	 *            proof.
	 */
	public void startProofCode(ArrayList<String> list) {
		list.add("\\begin{boxproof}\n");
	}

	/**
	 * @param list
	 *            is the list which contains the whole tex codes.and to this
	 *            list is added the tex code for ending the proof.
	 * 
	 */
	public void endProofCode(ArrayList<String> list) {

		list.add("\\end{boxproof}\n");

	}

	/**
	 * @param list
	 *            is the list which contains the whole tex codes.and to this
	 *            list is added the tex code for ending the transformed tex
	 *            file.
	 */
	public void endProofFile(ArrayList<String> list) {

		list.add("\\end{document}\n");

	}

}