import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author Suat
 * 
 *         Here i transform the given proof in tex code according to the package
 *         proof.sty. The idea is here transform line by line.
 * */
public class TreeTex {

	ArrayList<String> list = new ArrayList<String>();

	public TreeTex() {

	}

	// begin converting equality in approx
	/**
	 * @param equality
	 * 
	 *            is the proofline in the given proof file.
	 * @return
	 * 
	 *         returns the transformed tex code of every equality.the equality
	 *         detected using patter matching. the equality is between two
	 *         semicolons
	 */
	public String approx(String equality) {
		String approxString = null;
		Pattern approx = Pattern.compile(";.+;");

		Matcher matchApprox = approx.matcher(equality);
		while (matchApprox.find())
			approxString = "{"
					+ matchApprox.group(0).replaceAll("=", " \\\\approx ")
							.replaceAll(";", "") + "}";
		return approxString;

	}

	/**
	 * @param In
	 *            has the content of the given proof file.
	 * 
	 *            the method infer transforms the given proofs line by line tex
	 *            code according to the package proof.sty
	 */
	public void infer(ArrayList<String> In) {
		int lines = 1;
		for (String st : In) {
			if (st == "\n")
				continue;
			String rule = rulesParts(st)[1];
			String equality = st;

			// add a line more in to list in order to get the same size as array
			if (lines == 1)
				list.add("\n");
			lines++;

			if (rule.equals("prem"))
				list.add("\\infer" + "[(" + "app" + ")]" + approx(equality)
						+ "{}\n");

			else if (rule.equals("app"))
				list.add("\\infer" + "[(" + "app" + ")]" + approx(equality)
						+ "{}\n");
			else if (rule.equals("ref"))
				list.add("\\infer" + "[(" + "ref" + ")]" + approx(equality)
						+ "{}\n");
			else
				list.add("\\infer" + "[(" + rule + ")]" + approx(equality)
						+ "{\n");
		}

	}

	/**
	 * @param string
	 *            is the line which is in the given proof file.
	 * @return
	 * 
	 *         returns an array of a String.the array contains as follows:
	 *         rulesInParts[0]= is the number/index of every line
	 *         rulesInParts[1]= is the name of the rule..........
	 *         rulesInParts[2]= is the reference-number/argument of the rules
	 *         symetrity,application and congruence with a argument
	 *         rulesInParts[3]= the first argument of trans or congruence with
	 *         two arugments ..........................................
	 *         rulesInParts[4]= the second argument of trans or ............
	 *         congruence with two arugments ............................
	 *         rulesInParts[5]= is same of rulesInParts[1] ............
	 * 
	 */
	public String[] rulesParts(String string) {
		String[] rulesInParts = new String[7];
		// 0=linenum, 1=rule, 2=proof reference num. of all rules except trans,
		// 3=first
		// proofrefnum of trans, 4=secondproofrefnum of trans
		// find line number
		Pattern UnaryFirstNum = Pattern.compile("\\d+;");
		Matcher UnaryNum = UnaryFirstNum.matcher(string);
		while (UnaryNum.find())
			rulesInParts[0] = UnaryNum.group(0).replaceAll(";", "");

		// the name of the rule
		Pattern pattern = Pattern
				.compile("cong\\d+|sym\\d+|prem|app\\d+|app|ref|trans\\d+,\\d+|cong\\d+,\\d+");
		Matcher matcher = pattern.matcher(string);
		while (matcher.find())
			// if(matcher.group().equals("prem"))
			// plineRef[1]= matcher.group(0).replaceAll("prem", "app");
			// else
			rulesInParts[1] = matcher.group(0).replaceAll("\\d+|,", "");
		// rulesInParts[1] = matcher.group(0);
		// find proofline reference numbers except trans, and congruence with
		// two arguments. and of course prem and ref
		Pattern UnarySecondNum = Pattern
				.compile("cong\\d+|cong\\d+|app\\d+|sym\\d+");
		Matcher UnarySecNum = UnarySecondNum.matcher(string);
		while (UnarySecNum.find())
			rulesInParts[2] = UnarySecNum.group(0).replaceAll("sym|cong|app",
					"");

		// find proofline reference numbers for trans and congruence with tho
		// arguments
		Pattern trans = Pattern.compile("trans\\d+,\\d+|cong\\d+,\\d+");
		Matcher transpart = trans.matcher(string);
		while (transpart.find()) {

			// 11.03 added \\d?
			// plineTrans[0] = UnaryNum.group(0).replaceAll("\\d\\d?,\\d\\d?",
			// "");
			rulesInParts[3] = transpart.group(0).replaceAll("cong|trans|,\\d+",
					"");// firstnumber
			// of
			// trans
			rulesInParts[4] = transpart.group(0).replaceAll(
					"trans\\d+,|cong\\d+,", "");// secondnumber
			// of
			// trans

		}

		Pattern pattern2 = Pattern.compile("cong\\d+,\\d+,\\d+(,\\d+)*");
		Matcher matcher5 = pattern2.matcher(string);
		while (matcher5.find()) {
			// if(matcher.group().equals("prem"))
			// plineRef[1]= matcher.group(0).replaceAll("prem", "app");
			// else
			// rulesInParts[5] = matcher2.group(0).replaceAll("\\d+|,", "");
			rulesInParts[5] = matcher5.group(0).replaceAll(
					"cong|,\\d+,\\d+(,\\d+)*", "");
			rulesInParts[6] = matcher5.group(0)
					.replaceAll("cong\\d+,\\d+,", "");
		}
		return rulesInParts;

	}

	/**
	 * @param listIn
	 *            has the content of the the given proof file.
	 * @return
	 * 
	 *         returns an array of the transitivity and congruence rules if they
	 *         exists.
	 */
	public ArrayList<String> findTrans(ArrayList<String> listIn) {
		ArrayList<String> listTemp = new ArrayList<String>();
		String[] array = new String[3];

		for (String s : listIn) {
			if (s == "\n")
				continue;
			array = s.split(";");
			if (array[2].matches("\\s*trans\\d+,\\d+\\s*")
					|| array[2].matches("\\s*cong\\d+,\\d+\\s*")
					|| array[2].matches("\\s*cong\\d+,\\d+(,\\d+)+\\s*"))
				listTemp.add(s);
		}
		return listTemp;
	}

	/**
	 * @param n
	 *            is the number of the curly closing braces.
	 * @return
	 * 
	 *         returns the curly closing braces as strings
	 */
	public String close(int n) {
		String zu = "";
		while (n > 0) {
			zu += "}";
			n--;
		}

		return zu;
	}

	// begin needed commands for latex file
	/**
	 * @param list
	 * 
	 *            is the array which contains all tex codes, and here is added
	 *            the needed tex code for starting of the proof .
	 */
	public void startProofCode(ArrayList<String> list) {
		list.add("$$\n");
	}

	/**
	 * @param list
	 * 
	 *            is the array which contains all tex codes, and here is added
	 *            the needed tex code for ending of the proof .
	 */
	public void endProofCode(ArrayList<String> list) {

		list.add("$$\n");

	}

	/**
	 * @param list
	 * 
	 *            is the array which contains all tex codes, and here is added
	 *            the needed tex code for starting of proof file.
	 */
	public void startProofFile(ArrayList<String> list) {
		list.add("\\documentclass{ article}\n\\usepackage{proof}\n\\usepackage{amsmath}\n\\begin{document}\n");

	}

	/**
	 * @param list
	 *            is the array which contains all tex codes, and here is added
	 *            the needed tex code for ending of proof file.
	 */
	public void endProofFile(ArrayList<String> list) {

		list.add("\\end{document}");

	}

	// end needed commands for latex file

}
