import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Suat
 * 
 *         The implementation of transforming in tree-view using TreeTex.java .
 *         here we differ in base-case and in step-case. 1-Base-case is the case
 *         where there is no node. 2-Step-case is the case where there are nodes
 *         like transitivity or congruence with two parameters. And in step-case
 *         we differ again the types of the nodes. e.g.: if the paraters of a
 *         node is again a node or not.
 * 
 * */

public class TreeView {

	public TreeView() {

	}

	ArrayList<String> listOut = new ArrayList<String>();
	TreeTex tex = new TreeTex();

	/**
	 * @param args
	 *            is an arraylist which contains the content of the given proof
	 *            file.
	 * 
	 *            we have here basecase and stepcase......................
	 *            basecase: a proof which does not contain a node.........
	 *            E.g.:no transitivity no congruence with two arguments
	 * 
	 *            stepcase: a proof tree that contains arbitrary nodes.......
	 *            E.g.: the occurrences of nodes is serval times.............
	 * 
	 *            and every node has also four cases........................
	 *            case 1:the successors of node are none node..................
	 *            case 2:the left side of the node is a node and the other side
	 *            not..........................................................
	 *            case 3:the right side of the node is a node and the other side
	 *            not
	 *            ..............................................................
	 *            case 4:the successors of the node are again nodes....
	 * 
	 * 
	 * 
	 * 
	 * @throws IOException
	 */
	public TreeView(ArrayList<String> listIn) {
		// convert line by line in tex code and save in tex.list
		tex.infer(listIn);
		ArrayList<String> listBase = new ArrayList<String>();
		Map<Integer, ArrayList<String>> map = new HashMap<Integer, ArrayList<String>>();
		boolean switchAlgorithm = true;
		// basecase: no node
		if (tex.findTrans(listIn).isEmpty()) {
			int close = 0;
			for (int s = tex.list.size() - 1; s >= 1; s--) {

				// start for prem app
				if (listIn.get(s).split(";")[2].contains("app")
						&& listIn
								.get(Integer.parseInt(tex.rulesParts(listIn
										.get(s))[2])).contains("prem")) {
					listBase.add(tex.list.get(s));
					break;

				}
				// end prem app
				else if (listIn.get(s).contains("prem")
						|| listIn.get(s).contains("ref")
						|| listIn.get(s).contains("app")) {
					listBase.add(tex.list.get(s));
					break;
				}
				listBase.add(tex.list.get(s));

				close++;

			}

			listBase.add(tex.close(close));

		} else
			// stepcase
			for (int t = 0; t < tex.findTrans(listIn).size(); t++) {

				ArrayList<String> subList = new ArrayList<String>();
				int l = 0;
				int r = 0;
				int l2 = 0;
				int r2 = 0;
				int l3 = 0;
				int r3 = 0;
				int l4 = 0;
				int r4 = 0;
				int l5 = 0;

				// find plines of trans
				int pl = Integer.parseInt(tex.rulesParts(tex.findTrans(listIn)
						.get(t))[0]);

				if (tex.findTrans(listIn).get(t).split(";")[2]
						.matches("\\s*trans\\d+,\\d+\\s*")
						|| tex.findTrans(listIn).get(t).split(";")[2]
								.matches("\\s*cong\\d+,\\d+\\s*")) {
					// first trans pline
					l = Integer.parseInt(tex.rulesParts(listIn.get(pl))[3]);
					// second trans pline
					r = Integer.parseInt(tex.rulesParts(listIn.get(pl))[4]);
					// add node trans in list
				} else if (tex.findTrans(listIn).get(t).split(";")[2]
						.matches("\\s*cong\\d+,\\d+,\\d+\\s*")) {
					// third cong pline
					// first trans pline
					l = Integer.parseInt(tex.rulesParts(listIn.get(pl))[5]);
					// second trans pline
					r = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[1]);
					// add node trans in list
					l2 = Integer.parseInt(tex.rulesParts(listIn.get(pl))[6]);
				} else if (tex.findTrans(listIn).get(t).split(";")[2]
						.matches("\\s*cong\\d+,\\d+,\\d+,\\d+\\s*")) {
					// third cong pline
					// first trans pline
					l = Integer.parseInt(tex.rulesParts(listIn.get(pl))[5]);
					// second trans pline
					r = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[1]);
					// add node trans in list
					l2 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[2]);
					r2 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[3]);

				} else if (tex.findTrans(listIn).get(t).split(";")[2]
						.matches("\\s*cong\\d+,\\d+,\\d+,\\d+,\\d+\\s*")) {
					// third cong pline
					// first trans pline
					l = Integer.parseInt(tex.rulesParts(listIn.get(pl))[5]);
					// second trans pline
					r = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[1]);
					// add node trans in list
					l2 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[2]);
					r2 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[3]);
					l3 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[4]);
				} else if (tex.findTrans(listIn).get(t).split(";")[2]
						.matches("\\s*cong\\d+,\\d+,\\d+,\\d+,\\d+,\\d+\\s*")) {
					// third cong pline
					// first trans pline
					l = Integer.parseInt(tex.rulesParts(listIn.get(pl))[5]);
					// second trans pline
					r = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[1]);
					// add node trans in list
					l2 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[2]);
					r2 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[3]);
					l3 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[4]);
					r3 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[5]);
				} else if (tex.findTrans(listIn).get(t).split(";")[2]
						.matches("\\s*cong\\d+,\\d+,\\d+,\\d+,\\d+,\\d+,\\d+\\s*")) {
					// third cong pline
					// first trans pline
					l = Integer.parseInt(tex.rulesParts(listIn.get(pl))[5]);
					// second trans pline
					r = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[1]);
					// add node trans in list
					l2 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[2]);
					r2 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[3]);
					l3 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[4]);
					r3 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[5]);
					l4 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[6]);
				} else if (tex.findTrans(listIn).get(t).split(";")[2]
						.matches("\\s*cong\\d+,\\d+,\\d+,\\d+,\\d+,\\d+,\\d+,\\d+\\s*")) {
					// third cong pline
					// first trans pline
					l = Integer.parseInt(tex.rulesParts(listIn.get(pl))[5]);
					// second trans pline
					r = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[1]);
					// add node trans in list
					l2 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[2]);
					r2 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[3]);
					l3 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[4]);
					r3 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[5]);
					l4 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[6]);
					r4 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[7]);
				} else if (tex.findTrans(listIn).get(t).split(";")[2]
						.matches("\\s*cong\\d+,\\d+,\\d+,\\d+,\\d+,\\d+,\\d+,\\d+,\\d+\\s*")) {
					// third cong pline
					// first trans pline
					l = Integer.parseInt(tex.rulesParts(listIn.get(pl))[5]);
					// second trans pline
					r = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[1]);
					// add node trans in list
					l2 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[2]);
					r2 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[3]);
					l3 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[4]);
					r3 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[5]);
					l4 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[6]);
					r4 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[7]);
					l5 = Integer.parseInt(listIn.get(pl).split(";")[2]
							.split(",")[8]);
				}

				subList.add(tex.list.get(pl));
				int closeL = 0;// tex.close(closeL);leftside of a node
				int closeR = 0; // tex.close(closeR);rightside of a node

				/**
				 * The algorithm is the default which supports congruence with
				 * at most nine arguments. By default it is enabled, if you want
				 * to disable it please set on switchAlgo=false;
				 * 
				 * 
				 * 
				 * */

				if (switchAlgorithm == true) {
					// beginn of leftside
					// 27.05 break if trans. do till trans
					// notWorked:Integer.parseInt(listIn.get(l).split(";")[2].split("app")[1]);
					// worked:Integer.parseInt(tex.rulesParts(listIn.get(l))[2])
					while (true) {
						// start for prem app
						if (listIn.get(l).split(";")[2].contains("app")
								&& listIn.get(
										Integer.parseInt(tex.rulesParts(listIn
												.get(l))[2])).contains("prem")) {
							subList.add(tex.list.get(l));
							break;

						}
						// end prem app
						else if (listIn.get(l).contains("prem")
								|| listIn.get(l).contains("ref")) {
							subList.add(tex.list.get(l));
							break;
						}
						if (listIn.get(l).contains("trans")
								|| listIn.get(l).split(";")[2]
										.matches("\\s*cong\\d+,\\d+(,\\d+)*\\s*")) {
							for (String s : map.get(l))
								subList.add(s);
							break;
						}

						subList.add(tex.list.get(l));
						l = Integer.parseInt(tex.rulesParts(listIn.get(l))[2]);
						closeL++;
					}

					subList.add(tex.close(closeL));
					// end of leftside
					int arguments = 2;

					do {
						subList.add("&\n");
						// begin of rightside

						while (true) {
							// start for prem app
							if (listIn.get(r).split(";")[2].contains("app")
									&& listIn
											.get(Integer.parseInt(tex
													.rulesParts(listIn.get(r))[2]))
											.contains("prem")) {
								subList.add(tex.list.get(r));
								break;

							}
							// end prem app
							else if (listIn.get(r).contains("prem")
									|| listIn.get(r).contains("ref")) {
								subList.add(tex.list.get(r));
								break;
							}
							if (listIn.get(r).contains("trans")
									|| listIn.get(r).split(";")[2]
											.matches("\\s*cong\\d+,\\d+(,\\d+)*\\s*")) {
								for (String s : map.get(r))
									subList.add(s);
								break;
							}
							subList.add(tex.list.get(r));
							r = Integer
									.parseInt(tex.rulesParts(listIn.get(r))[2]);
							closeR++;
						}

						subList.add(tex.close(closeR));
						arguments++;
						if (arguments == 3)
							r = l2;
						else if (arguments == 4)
							r = r2;
						else if (arguments == 5)
							r = l3;
						else if (arguments == 6)
							r = r3;
						else if (arguments == 7)
							r = l4;
						else if (arguments == 8)
							r = r4;
						else if (arguments == 9)
							r = l5;

						if (arguments > tex.findTrans(listIn).get(t).split(";")[2]
								.split(",").length)
							break;
					} while (tex.findTrans(listIn).get(t).split(";")[2]
							.matches("\\s*cong\\d+,\\d+(,\\d+)+\\s*"));
					subList.add("}\n");
					// end of rightside
					// every node(trans) is in map
					map.put(pl, subList);
				}
				/**
				 * The algorithm which supports congruence with at most two
				 * arguments. By default it is disabled, if you want to enable
				 * it please set switchAlgo=true;
				 * 
				 * 
				 * 
				 * */

				if (switchAlgorithm == false)
					if (!listIn.get(l).contains("trans")
							&& !listIn.get(r).contains("trans")
							|| !listIn.get(l).split(";")[2]
									.matches("\\s*cong\\d+,\\d+\\s*")
							&& !listIn.get(r).split(";")[2]
									.matches("\\s*cong\\d+,\\d+\\s*")) {

						// beginn of leftside
						// 27.05 break if trans. do till trans
						// notWorked:Integer.parseInt(listIn.get(l).split(";")[2].split("app")[1]);
						// worked:Integer.parseInt(tex.rulesParts(listIn.get(l))[2])
						while (true) {
							// start for prem app
							if (listIn.get(l).split(";")[2].contains("app")
									&& listIn
											.get(Integer.parseInt(tex
													.rulesParts(listIn.get(l))[2]))
											.contains("prem")) {
								subList.add(tex.list.get(l));
								break;

							}
							// end prem app
							else if (listIn.get(l).contains("prem")
									|| listIn.get(l).contains("ref")) {
								subList.add(tex.list.get(l));
								break;
							}
							if (listIn.get(l).contains("trans")
									|| listIn.get(l).split(";")[2]
											.matches("\\s*cong\\d+,\\d+\\s*")) {
								for (String s : map.get(l))
									subList.add(s);
								break;
							}

							subList.add(tex.list.get(l));
							l = Integer
									.parseInt(tex.rulesParts(listIn.get(l))[2]);
							closeL++;
						}

						subList.add(tex.close(closeL));
						// end of leftside
						subList.add("&\n");
						// begin of rightside

						while (true) {
							// start for prem app
							if (listIn.get(r).split(";")[2].contains("app")
									&& listIn
											.get(Integer.parseInt(tex
													.rulesParts(listIn.get(r))[2]))
											.contains("prem")) {
								subList.add(tex.list.get(r));
								break;

							}
							// end prem app
							else if (listIn.get(r).contains("prem")
									|| listIn.get(r).contains("ref")) {
								subList.add(tex.list.get(r));
								break;
							}
							if (listIn.get(r).contains("trans")
									|| listIn.get(r).split(";")[2]
											.matches("\\s*cong\\d+,\\d+\\s*")) {
								for (String s : map.get(r))
									subList.add(s);
								break;
							}
							subList.add(tex.list.get(r));
							r = Integer
									.parseInt(tex.rulesParts(listIn.get(r))[2]);
							closeR++;
						}

						subList.add(tex.close(closeR));
						subList.add("}\n");
						// end of rightside
						// every node(trans) is in map
						map.put(pl, subList);

					}

					else if (!listIn.get(l).contains("trans")
							&& listIn.get(r).contains("trans")
							|| !listIn.get(l).split(";")[2]
									.matches("\\s*cong\\d+,\\d+\\s*")
							&& listIn.get(r).split(";")[2]
									.matches("\\s*cong\\d+,\\d+\\s*")
							|| !listIn.get(l).contains("trans")
							&& listIn.get(r).split(";")[2]
									.matches("\\s*cong\\d+,\\d+\\s*")
							|| !listIn.get(l).split(";")[2]
									.matches("\\s*cong\\d+,\\d+\\s*")
							&& listIn.get(r).contains("trans")) {
						// start leftside calculate

						while (true) {

							// start for prem app
							if (listIn.get(l).split(";")[2].contains("app")
									&& listIn
											.get(Integer.parseInt(tex
													.rulesParts(listIn.get(l))[2]))
											.contains("prem")) {
								subList.add(tex.list.get(l));
								break;

							}
							// end prem app
							else if (listIn.get(l).contains("prem")
									|| listIn.get(l).contains("ref")) {
								subList.add(tex.list.get(l));
								break;
							}
							if (listIn.get(l).contains("trans")
									|| listIn.get(l).split(";")[2]
											.matches("\\s*cong\\d+,\\d+\\s*")) {
								for (String s : map.get(l))
									subList.add(s);
								break;
							}
							subList.add(tex.list.get(l));
							l = Integer
									.parseInt(tex.rulesParts(listIn.get(l))[2]);
							closeL++;
						}

						subList.add("&\n");
						for (String sr : map.get(r))
							subList.add(sr);
						subList.add("}");
						map.put(pl, subList);
					}

					else if (listIn.get(l).contains("trans")
							&& !listIn.get(r).contains("trans")

							|| listIn.get(l).contains("trans")
							&& !listIn.get(r).split(";")[2]
									.matches("\\s*cong\\d+,\\d+\\s*")

							|| listIn.get(l).split(";")[2]
									.matches("\\s*cong\\d+,\\d+\\s*")
							&& !listIn.get(r).contains("trans")

							|| listIn.get(l).split(";")[2]
									.matches("\\s*cong\\d+,\\d+\\s*")
							&& !listIn.get(r).split(";")[2]
									.matches("\\s*cong\\d+,\\d+\\s*")) {

						for (String sl : map.get(l))
							subList.add(sl);
						subList.add("&\n");

						// start rightside calculate
						while (true) {

							// start for prem app
							if (listIn.get(r).split(";")[2].contains("app")
									&& listIn
											.get(Integer.parseInt(tex
													.rulesParts(listIn.get(r))[2]))
											.contains("prem")) {
								subList.add(tex.list.get(r));
								break;

							}
							// end prem app
							else if (listIn.get(r).contains("prem")
									|| listIn.get(r).contains("ref")) {
								subList.add(tex.list.get(r));
								break;
							}

							if (listIn.get(r).contains("trans")
									|| listIn.get(r).split(";")[2]
											.matches("\\s*cong\\d+,\\d+\\s*")) {
								for (String s : map.get(r))
									subList.add(s);
								break;
							}
							subList.add(tex.list.get(r));
							r = Integer
									.parseInt(tex.rulesParts(listIn.get(r))[2]);
							closeR++;
						}
						// end
						subList.add("}");
						map.put(pl, subList);

					}

					else if (listIn.get(l).contains("trans")
							&& listIn.get(r).contains("trans")

							|| listIn.get(l).split(";")[2]
									.matches("\\s*cong\\d+,\\d+\\s*")
							&& listIn.get(r).split(";")[2]
									.matches("\\s*cong\\d+,\\d+\\s*")

							|| listIn.get(l).contains("trans")
							&& listIn.get(r).split(";")[2]
									.matches("\\s*cong\\d+,\\d+\\s*")

							|| listIn.get(l).split(";")[2]
									.matches("\\s*cong\\d+,\\d+\\s*")
							&& listIn.get(r).contains("trans")) {

						for (String sl : map.get(l))
							subList.add(sl);

						subList.add("&\n");
						for (String sr : map.get(r))
							subList.add(sr);
						subList.add("}");

						map.put(pl, subList);

					}
			} // end of else/for

		tex.startProofFile(listOut);
		tex.startProofCode(listOut);
		if (!tex.findTrans(listIn).isEmpty()) {
			int par = 0;
			int transLine = Integer.parseInt(tex.rulesParts(tex.findTrans(
					listIn).get(tex.findTrans(listIn).size() - 1))[0]);
			int lastLine = listIn.size() - 1;
			ArrayList<String> beforTrans = new ArrayList<String>();
			if (transLine != lastLine)
				while (!listIn.get(lastLine).contains("trans")
						&& !listIn.get(lastLine).split(";")[2]
								.matches("\\s*cong\\d+,\\d+\\s*")
						&& !listIn.get(lastLine).split(";")[2]
								.matches("\\s*cong\\d+,\\d+,\\d+\\s*")
						&& !listIn.get(lastLine).split(";")[2]
								.matches("\\s*cong\\d+,\\d+,\\d+(,\\d+)+\\s*")) {

					beforTrans.add(tex.list.get(lastLine));
					lastLine = Integer.parseInt(tex.rulesParts(listIn
							.get(lastLine))[2]);
					par++;
				}

			if (transLine != listIn.size() - 1)
				for (String s : beforTrans)
					listOut.add(s);
			// rulesparts[0] is proofline, add every node in from down to top
			for (String s : map.get(Integer.parseInt(tex.rulesParts(tex
					.findTrans(listIn).get(tex.findTrans(listIn).size() - 1)

			)[0]

			)

			)

			)
				listOut.add(s);

			if (transLine != listIn.size() - 1) {
				listOut.add(tex.close(par));
				listOut.add("\n");

			}
		} else
			for (String s : listBase)
				listOut.add(s);
		tex.endProofCode(listOut);
		tex.endProofFile(listOut);

	}
}